# Generated by roxygen2: do not edit by hand

S3method(as.character,h5)
S3method(print,compress)
S3method(print,h5)
S3method(print,inspect)
S3method(str,h5)
export(h5_attr_names)
export(h5_class)
export(h5_compression)
export(h5_create_file)
export(h5_create_group)
export(h5_delete)
export(h5_dim)
export(h5_exists)
export(h5_inspect)
export(h5_is_dataset)
export(h5_is_group)
export(h5_length)
export(h5_ls)
export(h5_move)
export(h5_names)
export(h5_open)
export(h5_read)
export(h5_str)
export(h5_typeof)
export(h5_write)
useDynLib(h5lite, .registration = TRUE)
