% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect.r
\name{h5_inspect}
\alias{h5_inspect}
\title{Inspect HDF5 Dataset Creation Properties}
\usage{
h5_inspect(file, name)
}
\arguments{
\item{file}{The path to the HDF5 file.}

\item{name}{The full path of the dataset to inspect.}
}
\value{
An object of class \code{inspect} (a named list) containing:
\item{layout}{A string indicating storage layout (e.g., "chunked", "contiguous").}
\item{chunk_dims}{A numeric vector of chunk dimensions, or \code{NULL} if not chunked.}
\item{filters}{A list describing each filter applied.}
}
\description{
Retrieves the Dataset Creation Property List (DCPL) details including storage
layout, chunk dimensions, and a detailed list of all applied filters.
}
\examples{
file <- tempfile(fileext = ".h5")

compress <- h5_compression('lz4-9', int_packing = TRUE, checksum = TRUE)
h5_write(matrix(5001:5100, 10, 10), file, "packed_mtx", compress = compress)
h5_inspect(file, "packed_mtx")

mtx <- matrix(rnorm(1000), 100, 10)
h5_write(mtx, file, "float_mtx", compress = 'blosc2-zfp-prec-3')
res <- h5_inspect(file, "float_mtx")
print(res)

# Print the raw cd_values for blosc2
dput(res$filters[[1]]$cd_values)

unlink(file)
}
